/*
PROducer RGB Firmware
VERSION 1.32
[c]2014 DCR Dirk Conrad

DDR[x]  ... Datenrichtung (0=Eingang; 1 = Ausgang)
PORT[x] ... Ausgabe auf Port oder 1= PullUp Widerstand bei Datenrichtung einlesen
PIN[x]  ... Einlesen vom Port
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <compat/ina90.h>
#include <stdio.h>
#include <util/twi.h>
#include <avr/pgmspace.h>

// Def
#define I2C_SLA_ADR		0x50

// Sub
void Init (void);
void Sync_Write (void);
void Clear_Display (void);
void Draw_Workspace (void);
void Draw_RGB (unsigned char ucRd, unsigned char ucGn, unsigned char ucBl);
void RGB_Cal (void);
void I2C_Init(void);
void I2C_Start (void);
void I2C_Stop (void);
void I2C_Adr_W (unsigned char ucEESel);
void I2C_Adr_R (unsigned char ucEESel);
void I2C_Data_W (unsigned char ucByte);
unsigned char I2C_Data_R (void);
unsigned char I2C_Data_R_Ack (void);
void Wait_TWINT_Flag (void);
void Wait_TWSTO_Flag (void);
unsigned char EE_Read (unsigned long lAdr);
void EE_Write (unsigned long lAdr);
void USART_Init (void);
void USART_Transmit (char *data);
void USART_Receive (void);
void USART_Exec_Commands (void);
void Sample_ADC (void);

// Var
volatile unsigned char ucWorkspace[2][3][8] = { 
	{ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },

	{ { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } }
	};
volatile unsigned char ucDark[2] = { 0, 0 };
unsigned char ucTransition = 0;
unsigned char ucPaletteNum[2] = { 0, 0 };
volatile unsigned int uiSkipCounter = 0;
volatile unsigned int uiFirstSkipCounter = 0;
volatile unsigned long ulPos = 3;
volatile unsigned char ucRowCnt = 0;
volatile unsigned char ucColCnt = 0;
volatile unsigned char ucFrame = 0;
volatile unsigned char ucColorCnt = 0;
volatile unsigned char ucColor = 0;
volatile unsigned char ucCalRd = 10;
volatile unsigned char ucCalGn = 5;
volatile unsigned char ucCalBl = 3;
volatile unsigned char ucNewRow = 0;
volatile unsigned char ucDrawFrame = 1;
volatile unsigned char red[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
volatile unsigned char grn[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
volatile unsigned char blu[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
volatile unsigned char ucValRd = 0;
volatile unsigned char ucValGn = 0;
volatile unsigned char ucValBl = 0;
unsigned char ucEEWriteBuffer[128];
unsigned char sUSARTBuffer[136];
volatile int iUSARTBufferPointer = 0;
volatile unsigned int uiUSARTBusyCounter = 0;
volatile unsigned char ucMaster = 0;
volatile unsigned char ucSyncWrSeq = 0;
volatile unsigned char ucSyncWrPnt = 0;
volatile unsigned char ucSyncHiCnt = 0;
volatile unsigned char ucSyncLoCnt = 0;
volatile unsigned char ucMode = 0;
volatile int iADCValue = 512;
volatile int iADCSample[32] = { 0, 0, 0, 0, 0, 0, 0 ,0, 0, 0, 0, 0, 0, 0, 0 ,0, 0, 0, 0, 0, 0, 0, 0 ,0, 0, 0, 0, 0, 0, 0, 0 ,0 };
volatile unsigned char ucADCSamplePnt = 0;
volatile int iADCAudio[8] = { 0, 0, 0, 0, 0, 0, 0 ,0 };
volatile unsigned char iADCAudioPnt = 0;
unsigned char ucVUValue[8] = { 0, 0, 0, 0, 0, 0, 0 ,0 };
unsigned char ucVUHoldCnt[8] = { 0, 0, 0, 0, 0, 0, 0 ,0 };
volatile unsigned char ucAudioModCnt = 0;
volatile unsigned char ucAudioSmpCnt = 0;
volatile unsigned int uiSleepModCnt = 0;
volatile unsigned char ucSleepFlag = 0;
unsigned char ucVUPalette = 17;
unsigned char ucVUOrderNum = 0;
unsigned char ucUVMode = 0;
unsigned int uiVUPaletteDiv = 0;
unsigned char ucAudioDivCnt = 1;
volatile unsigned char ucGain = 0;
volatile unsigned char ucGainDiv = 80;
volatile unsigned char ucAudioEn = 0;
unsigned char ucFade = 0;
unsigned char ucFadeIn = 0;
unsigned char ucFadeOut = 0;
volatile unsigned long ulWait = 0;
volatile unsigned char ucClearFlag = 0;

// Lookup Table
unsigned char ucMulti[11][16] = {
	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  0 }, 
	{ 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,  1,  1,  1,  1,  1,  2 },
	{ 0, 0, 0, 1, 1, 1, 1, 1, 2, 2,  2,  2,  2,  3,  3,  3 },
	{ 0, 0, 1, 1, 1, 2, 2, 2, 2, 3,  3,  3,  4,  4,  4,  5 },
	{ 0, 0, 1, 1, 2, 2, 2, 3, 3, 4,  4,  4,  5,  5,  6,  6 },
	{ 0, 1, 1, 2, 2, 3, 3, 4, 4, 5,  5,  6,  6,  7,  7,  8 },
	{ 0, 1, 1, 2, 2, 3, 4, 4, 5, 5,  6,  7,  7,  8,  8,  9 },
	{ 0, 1, 1, 2, 3, 4, 4, 5, 6, 6,  7,  8,  8,  9, 10, 11 },
	{ 0, 1, 2, 2, 3, 4, 5, 6, 6, 7,  8,  9, 10, 10, 11, 12 },
	{ 0, 1, 2, 3, 4, 5, 5, 6, 7, 8,  9, 10, 11, 12, 13, 14 },
	{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 }
};

// Palette
const unsigned char ucPalette[64][21] PROGMEM = { 
	{ 0xf,0x0,0x0,  0xf,0x5,0x0,  0xf,0xa,0x0,  0xf,0xf,0x0,  0xa,0xf,0x0,  0x5,0xf,0x0,  0x0,0xf,0x0 }, 
	{ 0xf,0x5,0x0,  0xf,0xa,0x0,  0xf,0xf,0x0,  0xa,0xf,0x0,  0x5,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x5 }, 
	{ 0xf,0xa,0x0,  0xf,0xf,0x0,  0xa,0xf,0x0,  0x5,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x5,  0x0,0xf,0xa }, 
	{ 0xf,0xf,0x0,  0xa,0xf,0x0,  0x5,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x5,  0x0,0xf,0xa,  0x0,0xf,0xf }, 
	{ 0xa,0xf,0x0,  0x5,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x5,  0x0,0xf,0xa,  0x0,0xf,0xf,  0x0,0xa,0xf }, 
	{ 0x5,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x5,  0x0,0xf,0xa,  0x0,0xf,0xf,  0x0,0xa,0xf,  0x0,0x5,0xf }, 
	{ 0x0,0xf,0x0,  0x0,0xf,0x5,  0x0,0xf,0xa,  0x0,0xf,0xf,  0x0,0xa,0xf,  0x0,0x5,0xf,  0x0,0x0,0xf }, 
	{ 0x0,0xf,0x5,  0x0,0xf,0xa,  0x0,0xf,0xf,  0x0,0xa,0xf,  0x0,0x5,0xf,  0x0,0x0,0xf,  0x5,0x0,0xf }, 
	{ 0x0,0xf,0xa,  0x0,0xf,0xf,  0x0,0xa,0xf,  0x0,0x5,0xf,  0x0,0x0,0xf,  0x5,0x0,0xf,  0xa,0x0,0xf }, 
	{ 0x0,0xf,0xf,  0x0,0xa,0xf,  0x0,0x5,0xf,  0x0,0x0,0xf,  0x5,0x0,0xf,  0xa,0x0,0xf,  0xf,0x0,0xf }, 
	{ 0x0,0xa,0xf,  0x0,0x5,0xf,  0x0,0x0,0xf,  0x5,0x0,0xf,  0xa,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0xa }, 
	{ 0x0,0x5,0xf,  0x0,0x0,0xf,  0x5,0x0,0xf,  0xa,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0xa,  0xf,0x0,0x5 }, 
	{ 0x0,0x0,0xf,  0x5,0x0,0xf,  0xa,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0xa,  0xf,0x0,0x5,  0xf,0x0,0x0 }, 
	{ 0x5,0x0,0xf,  0xa,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0xa,  0xf,0x0,0x5,  0xf,0x0,0x0,  0xf,0x5,0x0 }, 
	{ 0xa,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0xa,  0xf,0x0,0x5,  0xf,0x0,0x0,  0xf,0x5,0x0,  0xf,0xa,0x0 }, 
	{ 0xf,0x0,0xf,  0xf,0x0,0xa,  0xf,0x0,0x5,  0xf,0x0,0x0,  0xf,0x5,0x0,  0xf,0xa,0x0,  0xf,0xf,0x0 }, 
	{ 0xf,0x0,0xa,  0xf,0x0,0x5,  0xf,0x0,0x0,  0xf,0x5,0x0,  0xf,0xa,0x0,  0xf,0xf,0x0,  0xa,0xf,0x0 }, 
	{ 0xf,0x0,0x5,  0xf,0x0,0x0,  0xf,0x5,0x0,  0xf,0xa,0x0,  0xf,0xf,0x0,  0xa,0xf,0x0,  0x5,0xf,0x0 }, 
	{ 0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x8,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x8,  0x0,0xf,0xf }, 
	{ 0xf,0x8,0x0,  0xf,0xf,0x0,  0x8,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x8,  0x0,0xf,0xf,  0x0,0x8,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x8,  0x0,0xf,0xf,  0x0,0x8,0xf,  0x0,0x0,0xf }, 
	{ 0x8,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x8,  0x0,0xf,0xf,  0x0,0x8,0xf,  0x0,0x0,0xf,  0x8,0x0,0xf }, 
	{ 0x0,0xf,0x0,  0x0,0xf,0x8,  0x0,0xf,0xf,  0x0,0x8,0xf,  0x0,0x0,0xf,  0x8,0x0,0xf,  0xf,0x0,0xf }, 
	{ 0x0,0xf,0x8,  0x0,0xf,0xf,  0x0,0x8,0xf,  0x0,0x0,0xf,  0x8,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0x8 }, 
	{ 0x0,0xf,0xf,  0x0,0x8,0xf,  0x0,0x0,0xf,  0x8,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0x8,  0xf,0x0,0x0 }, 
	{ 0x0,0x8,0xf,  0x0,0x0,0xf,  0x8,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0x8,  0xf,0x0,0x0,  0xf,0x8,0x0 }, 
	{ 0x0,0x0,0xf,  0x8,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0x8,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0 }, 
	{ 0x8,0x0,0xf,  0xf,0x0,0xf,  0xf,0x0,0x8,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x8,0xf,0x0 }, 
	{ 0xf,0x0,0xf,  0xf,0x0,0x8,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x8,0xf,0x0,  0x0,0xf,0x0 }, 
	{ 0xf,0x0,0x8,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x8,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0x8 }, 
	{ 0xf,0xf,0x0,  0xf,0xf,0x5,  0xf,0xf,0xa,  0xf,0xf,0xf,  0xf,0xa,0xa,  0xf,0x5,0x5,  0xf,0x0,0x0 }, 
	{ 0x2,0x2,0x2,  0x4,0x4,0x4,  0x6,0x6,0x6,  0x9,0x9,0x9,  0xb,0xb,0xb,  0xd,0xd,0xd,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf }, 
	{ 0xf,0x8,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf,  0xf,0x0,0x0 }, 
	{ 0x0,0xf,0x0,  0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf,  0xf,0x0,0x0,  0xf,0x8,0x0 }, 
	{ 0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0 }, 
	{ 0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0 }, 
	{ 0xf,0x0,0xf,  0xf,0xf,0xf,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf }, 
	{ 0xf,0xf,0xf,  0xf,0x0,0x0,  0xf,0x8,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf,  0xf,0x0,0xf }, 
	{ 0xf,0x0,0x0,  0xf,0xf,0x0,  0x0,0xf,0x0,  0x0,0xf,0xf,  0x0,0x0,0xf,  0xf,0x0,0xf,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0x0,0xf,0xf,  0x0,0x8,0x8,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0xf,0xf,0x0,  0x8,0x8,0x0,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0x0,0x0,  0x8,0x0,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0x0,  0x0,0x8,0x0,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0xf,0xf,0x0,  0x8,0x8,0x0,  0x0,0xf,0xf,  0x0,0x8,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0x0,0xf,0xf,  0x0,0x8,0x8,  0x0,0x0,0xf,  0x0,0x0,0x8,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }, 
	{ 0x0,0xf,0xf,  0x0,0x8,0x8,  0x0,0xf,0x0,  0x0,0x8,0x0,  0xf,0x0,0xf,  0x8,0x0,0x8,  0xf,0xf,0xf }
 }; 

// VU-Balken
unsigned char ucVUColor[8][3] = {
	{ 0xFF, 0x00, 0X00 },
	{ 0x00, 0xFF, 0X00 },
	{ 0xFF, 0xFF, 0X00 },
	{ 0x00, 0x00, 0xFF },
	{ 0x00, 0x00, 0xFF },
	{ 0xFF, 0x00, 0xFF },
	{ 0x00, 0xFF, 0xFF },
	{ 0xFF, 0xFF, 0xFF },
};
unsigned char ucVUOrder[2][8] = {
	{ 0, 1, 2, 3, 4, 5 ,6, 7 },
	{ 0, 7, 1, 6, 2, 5 ,3, 4 }
};

// ########## Hauptprogramm ##########
int main (void) {
int iX;
int iY;
unsigned char ucB;

	Init ();
	USART_Init ();
	I2C_Init ();
	RGB_Cal ();

	ucTransition = EE_Read (ulPos+24);
	ucPaletteNum[ucDrawFrame] = EE_Read (ulPos+25);
	uiFirstSkipCounter = (ucTransition & 0x3F);
	uiSkipCounter = uiFirstSkipCounter;

	// Enable Interrupts
	sei ();

	while (1) {
		// RS232
		USART_Receive ();
		USART_Exec_Commands ();

		// Audio berwachen
		if (uiUSARTBusyCounter == 0) {
			ucAudioDivCnt++;
			 ucAudioDivCnt &= 0x3F;
			if ((ucAudioDivCnt == 0 && ucMaster == 1) || ucMode == 1) {
				if (iADCValue != 32000) {
					iADCValue = 32000;
					Sample_ADC ();
				}
			}
		}

		// Bildablauf
		if (ucNewRow) {
			ucNewRow = 0;

			if (ucMode == 0) {
				if (ucFadeIn == 0) {
					if (uiSkipCounter == 0) {
						if (ucFadeOut == 0) {
							ulPos += 26;
							ucTransition = EE_Read (ulPos+24);
							uiSkipCounter = (ucTransition & 0x3F);
							if (ulPos == 131069 || uiSkipCounter == 63) {
								ulPos = 3;
								ucTransition = EE_Read (ulPos+24);
								uiSkipCounter = (ucTransition & 0x3F);
								RGB_Cal ();
							}
							uiSkipCounter <<= 2;
							if (uiSkipCounter == 6 ) { uiSkipCounter = 5; }
							if (uiSkipCounter == 0 ) { uiSkipCounter = 1; }
							ucFade = (ucTransition & 0xC0) >> 6; 
							switch (ucFade) {
								case 0: ucFadeIn =  0; ucFadeOut =  0; break;
								case 1: ucFadeIn = 15; ucFadeOut =  0; break;      
								case 2: ucFadeIn =  0; ucFadeOut = 15; break;      
								case 3: ucFadeIn = 15; ucFadeOut = 15; break; 
							}
						}
					}
					else {
						uiSkipCounter--;
					}
				}
				ucDark[ucDrawFrame] = 0;
				if (ucFadeIn > 0) {
					ucDark[ucDrawFrame] = ucFadeIn;
					ucFadeIn--;
				}
				else {
					if (ucFadeOut > 0 && uiSkipCounter == 0) {
						ucDark[ucDrawFrame] = 15 - ucFadeOut;
						ucFadeOut--;
					}
				}

				ucPaletteNum[ucDrawFrame] = EE_Read (ulPos+25);
				for (iX=0; iX<8; iX++) {
					ucWorkspace[ucDrawFrame][0][iX] = EE_Read (ulPos+iX);
					ucWorkspace[ucDrawFrame][1][iX] = EE_Read (ulPos+iX+8);
					ucWorkspace[ucDrawFrame][2][iX] = EE_Read (ulPos+iX+16);
				}
			}

			if (ucMode == 1) {
				// Bild lschen
				for (iX=0; iX<8; iX++) {
					ucWorkspace[ucDrawFrame][0][iX] = 0;
					ucWorkspace[ucDrawFrame][1][iX] = 0;
					ucWorkspace[ucDrawFrame][2][iX] = 0;
				}
				// Palette
				ucPaletteNum[ucDrawFrame] = ucVUPalette;

				// Palette wechseln
				uiVUPaletteDiv++;
				uiVUPaletteDiv &= 0x0FFF;
				if (uiVUPaletteDiv == 0) {
						ucVUOrderNum = 0;
						ucVUPalette = 0;
						ucUVMode = 0;
				}
				if (uiVUPaletteDiv == 1024) {
						ucVUOrderNum = 1;
						ucVUPalette = 10;
						ucUVMode = 0;
				}
				if (uiVUPaletteDiv == 2048) {
						ucVUOrderNum = 0;
						ucVUPalette = 20;
						ucUVMode = 0;
				}
				if (uiVUPaletteDiv == 3072) {
						ucVUOrderNum = 0;
						ucVUPalette = 14;
						ucUVMode = 1;
				}

				// Gain
				if (ucGainDiv > 0) {
					ucGainDiv--;
				}
				else {
					if (ucGain > 0) {
						ucGain--;
					}
					ucGainDiv = 80;
				}

				for (iX=0; iX<8; iX++) {
					if (ucVUHoldCnt[iX] > 0) { ucVUHoldCnt[iX]--; }
					if ((iADCAudio[iX] - ucGain) > ucVUValue[iX]) {
						ucVUValue[iX] = (iADCAudio[iX] - ucGain);
						ucVUHoldCnt[iX] = 3;
						if (ucVUValue[iX] > 9) {
							ucVUValue[iX] = 9;
						}
					}
					else {
						if (ucVUValue[iX] > 0 && ucVUHoldCnt[iX] == 0) {
							ucVUValue[iX]--;
							ucVUHoldCnt[iX] = 3;
						}
					}
				}

				ucB = 0x01;
				for (iX=0; iX<8; iX++) {
					for (iY=0; iY<8; iY++) {
						if ((8-iY < ucVUValue[iX] && ucUVMode == 0) || (8-iY == ucVUValue[iX] && ucUVMode == 1)) {
							ucWorkspace[ucDrawFrame][0][ucVUOrder[ucVUOrderNum][iY]] |= (ucVUColor[iY][0] & ucB);
							ucWorkspace[ucDrawFrame][1][ucVUOrder[ucVUOrderNum][iY]] |= (ucVUColor[iY][1] & ucB);
							ucWorkspace[ucDrawFrame][2][ucVUOrder[ucVUOrderNum][iY]] |= (ucVUColor[iY][2] & ucB);
						}
					}
					ucB <<= 1;
				}

			}
		}

		if (ucMode == 2) {
			_SLEEP ();
		}
	}
}

// ########## Initialisierung ##########
void Init (void) {
	// Disable JTAG
	MCUCSR |= 0x80;
	MCUCSR |= 0x80;
	
	// ENABLE SLEEP MODE IDLE
	MCUCR |= ((1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0));
	
	// Port B
	// 0x01 MASTER/SLAVE (IN)
	// 0x02 CLK (OUT)
	// 0x04 LE (OUT)
	// 0x08 SD_R (OUT)
	// 0x10 SD_G (OUT)
	// 0x20 SD_B (OUT)
	// 0x40 ---
	// 0x80 ---
	DDRB = 0x3E;
	PORTB = 0x01;

	// Port C
	// 0x01 A0 (OUT)
	// 0x02 A1 (OUT)
	// 0x04 A2 (OUT)
	// 0x08 A3 (OUT)
	// 0x10 SDA
	// 0x20 SCL
	// 0x40 ---
	// 0x80 ---
	DDRC = 0x0F;
	PORTC = 0x0F;

	// Port D
	// 0x01 RXD (IN)
	// 0x02 TXD (OUT)
	// 0x04 SYNC 1 (IN)
	// 0x08 STATUS (OUT)
	// 0x10 A4 (OUT)
	// 0x20 A5 (OUT)
	// 0x40 A6 (OUT)
	// 0x80 A7 (OUT)
	DDRD = 0xFA;
	PORTD = 0xF1;

	Draw_RGB (0, 0, 0);

	// MASTER/SLAVE Bestimmung (Wartezeit vor dem ersten Lesen)
	for (ulWait=0; ulWait < 30000; ulWait++) { _NOP (); }
	if (PINB & 0x01) { // MASTER/SLAVE
		ucMaster = 1;
		DDRD = 0xFE; // SYNC auf Ausgang schalten
		for (ulWait=0; ulWait < 600000; ulWait++) {
			_NOP ();
		}
	}

	// Init Timer 1
	TCCR1A |= 0x00;
	TCCR1B |= ((1<<WGM12)|(0<<CS12)|(1<<CS11)|(1<<CS10));	// Prescaler 64 und..
	if (ucMaster == 1) { OCR1A = 60; }							// Output Compare Register A (16 MHz : 64 : 60 = 4166 kHz)
	if (ucMaster == 0) { OCR1A = 5; }
	TIMSK |= (1<<OCIE1A);										// Interrupt Timer erlauben
}

// ########## Interrupt Timer 1 ##########
ISR (TIMER1_COMPA_vect) {
	// USART Busy Counter
	if (uiUSARTBusyCounter > 0) {
		if (ucClearFlag == 0) {
			Clear_Display ();
			ucClearFlag = 1;
		}
		uiUSARTBusyCounter--;
	}
	else {
		ucClearFlag = 0;

		if (ucMaster) {
			// Smples auswerten fr Spleep-Mode Umschaltung
			if (ucAudioEn) {
				if (iADCAudio[0] > 1) {
					ucSleepFlag = 0;
					uiSleepModCnt = 0;
					PORTD |= 0x08;
				}
				uiSleepModCnt++;		
				if (uiSleepModCnt > 2000) {
					if (ucSleepFlag) {
						ucMode = 2;
						ucSyncWrSeq = 0xFF;
						ucSyncWrPnt = 1;
						Clear_Display ();
					}
					uiSleepModCnt = 0;
					ucSleepFlag = 1;
					PORTD &= 0xF7;
				}
				// Aufwachen
				if (iADCAudio[0] > 4) {
					if (ucMode == 2) {
						ucMode = 0;
						ucSyncWrSeq = 0xFF;
						ucSyncWrPnt = 1;
						Clear_Display ();
					}
				}
				// Samples auswerten fr VU-Meter Umschaltung
				if (iADCAudio[0] > 1) {
					ucAudioSmpCnt++;
				}

				if (ucMode != 2) {
					ucAudioModCnt++;
					if (ucAudioModCnt > 100) {
						if (ucAudioSmpCnt > 20) {
							if (ucMode == 0) {
								ucMode = 1;
								ucGain = 0;
								ucGainDiv = 80;
								ucDark[0] = 0;
								ucDark[1] = 0;
								ucVUPalette = 17;
								uiVUPaletteDiv = 0;
								ucVUOrderNum = 0;
								ucUVMode = 0;
								ucSyncWrSeq = 0x00;
								ucSyncWrPnt = 1;		
							}
						}
						if (ucAudioSmpCnt < 5) {
							if (ucMode == 1) {
								ucMode = 0;
								ucSyncWrSeq = 0xFF;
								ucSyncWrPnt = 1;
								Clear_Display ();
							}
						}
						ucAudioModCnt = 0;
						ucAudioSmpCnt = 0;
					}
				}
				ucAudioEn = 0;
			}

			// Sync auf 1. Frame
			if (ucMode == 0) {
				if (ulPos == 3 && uiSkipCounter == uiFirstSkipCounter) {
					if (ucColorCnt == 0 && ucRowCnt == 0) {
						ucSyncWrSeq = 0xFF;
						ucSyncWrPnt = 1;
						Clear_Display ();
					}
				}
			}
			if (ucSyncWrPnt == 0) {
				if (ucMode == 2) {
					ucSyncWrSeq = 0xFF;
				}
				else {
					ucSyncWrSeq = 0x55;
				}
				ucSyncWrPnt = 1;
			}
			Sync_Write ();
			Draw_Workspace ();
		}
		else {
			if ((PIND & 0x04)) {
				if (ucSyncHiCnt < 255) { ucSyncHiCnt++; }
				ucSyncLoCnt = 0;
			}
			else {
				if (ucSyncLoCnt < 255) { ucSyncLoCnt++; }
				ucSyncHiCnt = 0;
			}

			if (ucSyncLoCnt == 1 || ucSyncHiCnt == 1) {
				Draw_Workspace ();	
			}

			if (ucSyncHiCnt > 40) {
				ucMode = 0;
				Clear_Display ();
				PORTD &= 0xF7;
			}
			if (ucSyncLoCnt > 40) {
				ucMode = 1;
				ucGain = 0;
				ucGainDiv = 80;
				ucDark[0] = 0;
				ucDark[1] = 0;
				ucVUPalette = 17;
				uiVUPaletteDiv = 0;
				ucVUOrderNum = 0;
				ucUVMode = 0;
				PORTD |= 0x08;
			}
		}
	}
}

// ########## Interrupt ADC ##########
ISR (ADC_vect) {
int iADC_L = 0;
int iADC_H = 0;
int iX;
unsigned char ucOvlCnt = 0;
	iADC_L = ADCL;
	iADC_H = ADCH;
	iADCValue = (iADC_H << 8) + iADC_L;
	iADCValue -= 512;
	if (iADCValue > 0) {
		iADCSample[ucADCSamplePnt] = iADCValue >> 4;
	}
	else {
		iADCSample[ucADCSamplePnt] = (0 - iADCValue) >> 4;
	}
	ucADCSamplePnt++;
	if (ucADCSamplePnt == 32) {
		ucADCSamplePnt = 0;
		iADCAudio[iADCAudioPnt] = 0;
		for (iX=0; iX<32; iX++) {
			if (iADCSample[iX] > 9) {
				ucOvlCnt++;
			}
			if (iADCSample[iX] > iADCAudio[iADCAudioPnt]) {
				iADCAudio[iADCAudioPnt] = iADCSample[iX];
			}
		}
		iADCAudioPnt++;
		iADCAudioPnt &= 0x07;
		if (iADCAudioPnt == 0) { ucAudioEn = 1; }
		if (ucOvlCnt > 15) {
			if (ucGain < 8) {
				ucGain++;
			}
		}
	}
}

// ########## Kommunikation mit dem Nachbarmodul ##########
void Sync_Write () {
	if (ucSyncWrPnt) {
		if (ucSyncWrSeq & ucSyncWrPnt) { PORTD |= 0x04; } else { PORTD &= 0xFB; }
		ucSyncWrPnt <<= 1;
	}
}

// ########## Bildinhalte lschen ##########
void Clear_Display () {
int iX;
	ulPos = 3;
	ucNewRow = 0;
	ucColorCnt = 0;
	ucRowCnt = 0;
	uiSkipCounter = uiFirstSkipCounter;
	ucFadeIn = 0;
	ucFadeOut = 0;
	PORTC |= 0x0F; // A0-A4 auf 1 setzen
	PORTD |= 0XF0; // A5-A7 auf 1 setzen
	for (iX=0; iX<8; iX++) {
		ucWorkspace[0][0][iX] = 0x00;
		ucWorkspace[0][1][iX] = 0x00;
		ucWorkspace[0][2][iX] = 0x00;
		ucWorkspace[1][0][iX] = 0x00;
		ucWorkspace[1][1][iX] = 0x00;
		ucWorkspace[1][2][iX] = 0x00;
		red[iX] = 0x00;
		grn[iX] = 0x00;
		blu[iX] = 0x00;
	}
	Draw_RGB (0, 0, 0);
}

// ########## Den Arbeitsbereich aufs Bild bringen ##########
void Draw_Workspace () {
unsigned char ucTmpColorCntRd;
unsigned char ucTmpColorCntGn;
unsigned char ucTmpColorCntBl;
unsigned char ucTmpColCnt;

	ucColorCnt++;
	if (ucColorCnt > 8) {
		ucColorCnt = 0;
		ucRowCnt--;
		ucRowCnt &= 0x07;
		PORTC |= 0x0F; // A0-A4 auf 1 setzen
		PORTD |= 0XF0; // A5-A7 auf 1 setzen
		switch (ucRowCnt) {
			case 0: PORTC &= 0xFE; break; // A0
			case 1: PORTC &= 0xFD; break; // A1
			case 2: PORTC &= 0xFB; break; // A2
			case 3: PORTC &= 0xF7; break; // A3
			case 4: PORTD &= 0xEF; break; // A4
			case 5: PORTD &= 0xDF; break; // A5
			case 6: PORTD &= 0xBF; break; // A6
			case 7: PORTD &= 0x7F; break; // A7
		}
		if (ucRowCnt == 7) {
			ucNewRow = 1;
			ucFrame ^= 0x01;
			ucDrawFrame ^= 0x01;
		}

		for (ucColCnt=0; ucColCnt<8; ucColCnt++) {
			ucColor = 0x00;
			ucTmpColCnt = 1 << ucColCnt;
			if (ucWorkspace[ucFrame][0][ucRowCnt] & ucTmpColCnt) { ucColor |= 0x01; }
			if (ucWorkspace[ucFrame][1][ucRowCnt] & ucTmpColCnt) { ucColor |= 0x02; }
			if (ucWorkspace[ucFrame][2][ucRowCnt] & ucTmpColCnt) { ucColor |= 0x04; }
			switch (ucColor) {
				case 0:
					red[ucColCnt] = 0x00;
					grn[ucColCnt] = 0x00;
					blu[ucColCnt] = 0x00;
					break;
				case 1:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 0]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 1]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 2]);
					break;
				case 2:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 3]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 4]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 5]);
					break;
				case 3:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 6]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 7]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 8]);
					break;
				case 4:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][ 9]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][10]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][11]);
					break;
				case 5:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][12]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][13]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][14]);
					break;
				case 6:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][15]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][16]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][17]);
					break;
				case 7:
					red[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][18]);
					grn[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][19]);
					blu[ucColCnt] = pgm_read_byte(&ucPalette[ucPaletteNum[ucFrame]][20]);
					break;
			}
			red[ucColCnt] = ucMulti[ucCalRd][red[ucColCnt]];
			grn[ucColCnt] = ucMulti[ucCalGn][grn[ucColCnt]];
			blu[ucColCnt] = ucMulti[ucCalBl][blu[ucColCnt]];
		}
	}
	
	if (ucColorCnt < 8) {
		ucTmpColorCntRd = ucColorCnt + ucMulti[ucCalRd][ucDark[ucFrame]];
		ucTmpColorCntGn = ucColorCnt + ucMulti[ucCalGn][ucDark[ucFrame]];
		ucTmpColorCntBl = ucColorCnt + ucMulti[ucCalBl][ucDark[ucFrame]];

		ucTmpColorCntRd <<= 1;
		ucTmpColorCntGn <<= 1;
		ucTmpColorCntBl <<= 1;

		ucValRd = 0x00;
		if (ucTmpColorCntRd < red[0]) { ucValRd |= 0x01; }
		if (ucTmpColorCntRd < red[1]) { ucValRd |= 0x02; }
		if (ucTmpColorCntRd < red[2]) { ucValRd |= 0x04; }
		if (ucTmpColorCntRd < red[3]) { ucValRd |= 0x08; }
		if (ucTmpColorCntRd < red[4]) { ucValRd |= 0x10; }
		if (ucTmpColorCntRd < red[5]) { ucValRd |= 0x20; }
		if (ucTmpColorCntRd < red[6]) { ucValRd |= 0x40; }
		if (ucTmpColorCntRd < red[7]) { ucValRd |= 0x80; }

		ucValGn = 0x00;
		if (ucTmpColorCntGn < grn[0]) { ucValGn |= 0x01; }
		if (ucTmpColorCntGn < grn[1]) { ucValGn |= 0x02; }
		if (ucTmpColorCntGn < grn[2]) { ucValGn |= 0x04; }
		if (ucTmpColorCntGn < grn[3]) { ucValGn |= 0x08; }
		if (ucTmpColorCntGn < grn[4]) { ucValGn |= 0x10; }
		if (ucTmpColorCntGn < grn[5]) { ucValGn |= 0x20; }
		if (ucTmpColorCntGn < grn[6]) { ucValGn |= 0x40; }
		if (ucTmpColorCntGn < grn[7]) { ucValGn |= 0x80; }

		ucValBl = 0x00;
		if (ucTmpColorCntBl < blu[0]) { ucValBl |= 0x01; }
		if (ucTmpColorCntBl < blu[1]) { ucValBl |= 0x02; }
		if (ucTmpColorCntBl < blu[2]) { ucValBl |= 0x04; }
		if (ucTmpColorCntBl < blu[3]) { ucValBl |= 0x08; }
		if (ucTmpColorCntBl < blu[4]) { ucValBl |= 0x10; }
		if (ucTmpColorCntBl < blu[5]) { ucValBl |= 0x20; }
		if (ucTmpColorCntBl < blu[6]) { ucValBl |= 0x40; }
		if (ucTmpColorCntBl < blu[7]) { ucValBl |= 0x80; }

		Draw_RGB (ucValRd, ucValGn, ucValBl);
	}
	if (ucColorCnt == 8) {
		Draw_RGB (0, 0, 0);
	}
}

// Schreibe auf LED Treiber
void Draw_RGB (unsigned char ucRd, unsigned char ucGn, unsigned char ucBl) {
	if (ucRd & 0x80) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x80) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x80) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x40) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x40) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x40) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x20) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x20) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x20) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x10) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x10) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x10) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x08) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x08) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x08) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x04) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x04) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x04) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x02) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x02) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x02) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	if (ucRd & 0x01) { PORTB |= 0x08; } else { PORTB &= 0xF7; } // SD_R
	if (ucGn & 0x01) { PORTB |= 0x10; } else { PORTB &= 0xEF; } // SD_G
	if (ucBl & 0x01) { PORTB |= 0x20; } else { PORTB &= 0xDF; } // SD_B
	PORTB |= 0x02; PORTB &= 0XFD; // CLK
	PORTB |= 0x04; PORTB &= 0XFB; // LE
}

// ########## RGB Kalibrierung ##########
void RGB_Cal (void) {
	// Kalibrierung Rot
	ucCalRd = EE_Read (0);
	// Kalibrierung Grn
	ucCalGn = EE_Read (1);
	// Kalibrierung Blau
	ucCalBl = EE_Read (2);
}

// ########## I2C Init ##########
void I2C_Init (void) {
	TWBR = 2;										// SCL Freq = 16MHz/(16+2*TWBR*4^TWSR) = 800KHz
	TWSR = (0<<TWPS1)|(0<<TWPS0);					// Teiler = 0 
}

// ########## I2C Start ##########
void I2C_Start (void) {
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);		// Send START condition
	Wait_TWINT_Flag ();
	if ((TWSR & 0xF8) != TW_START && (TWSR & 0xF8) != TW_REP_START) {	// Check value of TWI Status Register
		TWCR = 0;
	}
}

// ########## I2C Stop ##########
void I2C_Stop (void) {
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);		// Transmit STOP condition
	Wait_TWSTO_Flag ();
}

// ########## I2C Adr Write ##########
void I2C_Adr_W (unsigned char ucEESel) {
	TWDR = (I2C_SLA_ADR | ucEESel) << 1;
	TWCR = (1<<TWINT) | (1<<TWEN);					// Clear TWINT bit in TWCR to start transmission of address
	Wait_TWINT_Flag ();
	if ((TWSR & 0xF8) != TW_MT_SLA_ACK) {			// Check value of TWI Status Register
		TWCR = 0;	
	}
}

// ########## I2C Adr Read ##########
void I2C_Adr_R (unsigned char ucEESel) {
	TWDR = ((I2C_SLA_ADR | ucEESel) << 1) | 1;
	TWCR = (1<<TWINT) | (1<<TWEN);					// Clear TWINT bit in TWCR to start transmission of address
	Wait_TWINT_Flag ();
	if ((TWSR & 0xF8) != TW_MR_SLA_ACK) {			// Check value of TWI Status Register
		TWCR = 0;
	}
}

// ########## I2C Data Write ##########
void I2C_Data_W (unsigned char ucByte) {
	TWDR = ucByte;
	TWCR = (1<<TWINT) | (1<<TWEN);					// Clear TWINT bit in TWCR to start transmission of address
	Wait_TWINT_Flag ();
	if ((TWSR & 0xF8) != TW_MT_DATA_ACK) {			// Check value of TWI Status Register
		TWCR = 0;
	}
}

// ########## I2C Data Read NAck ##########
unsigned char I2C_Data_R (void) {
	TWCR = (1<<TWINT) | (1<<TWEN);					// Clear TWINT bit in TWCR to start transmission of address
	Wait_TWINT_Flag ();
	return TWDR;
}

// ########## I2C Data Read Ack ##########
unsigned char I2C_Data_R_Ack (void) {
	TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);	// Clear TWINT bit in TWCR to start transmission of address
	Wait_TWINT_Flag ();
	return TWDR;
}

// ########## I2C Wait for TWINT Flag ##########
void Wait_TWINT_Flag (void) {
unsigned int uiCnt = 0;
	while (!(TWCR & (1<<TWINT))) {
		uiCnt++;
		if (uiCnt == 1000) {
			return;
		}
	}
}

// ########## I2C Wait for TWSTO Flag ##########
void Wait_TWSTO_Flag (void) {
unsigned int uiCnt = 0;
	while (!(TWCR & (1<<TWSTO))) {
		uiCnt++;
		if (uiCnt == 1000) {
			return;
		}
	}
}

// ########## Lese aus EEPROM ##########
unsigned char EE_Read (unsigned long lAdr) {
unsigned char ucLOByte;
unsigned char ucHIByte;
unsigned char ucByte;
unsigned char ucEESel;
	ucLOByte = ((unsigned char *)&lAdr)[0];
	ucHIByte = ((unsigned char *)&lAdr)[1];
	ucEESel =  ((unsigned char *)&lAdr)[2];
	I2C_Start ();
	I2C_Adr_W (ucEESel);
	I2C_Data_W (ucHIByte);
	I2C_Data_W (ucLOByte);
	I2C_Start ();
	I2C_Adr_R (ucEESel);
	ucByte = I2C_Data_R ();
	I2C_Stop ();
	return ucByte;
}

// ########## Schreibe in EEPROM ##########
void EE_Write (unsigned long lAdr) {
unsigned char ucLOByte;
unsigned char ucHIByte;
unsigned char ucPt;
unsigned char ucEESel;
	ucLOByte = ((unsigned char *)&lAdr)[0];
	ucHIByte = ((unsigned char *)&lAdr)[1];
	ucEESel =  ((unsigned char *)&lAdr)[2];
	I2C_Start ();
	I2C_Adr_W (ucEESel);
	I2C_Data_W (ucHIByte);
	I2C_Data_W (ucLOByte);
	for (ucPt=0; ucPt<128; ucPt++) {
		I2C_Data_W (ucEEWriteBuffer[ucPt]);
	}
	I2C_Stop ();
}

// ########## USART Init ##########
void USART_Init (void) {
unsigned int ubrr;
	ubrr = 0;										// ubrr = ( FREQ / ( 16 * 1000k) ) -1
	UBRRH = (unsigned char)(ubrr>>8);				// Baud-Rate
	UBRRL = (unsigned char)ubrr;
	UCSRA = (0<<U2X);								// USART 2x Speed
	UCSRB = (1<<RXEN)|(1<<TXEN);					// Enable Receiver und Transmitter
	UCSRC = (1<<URSEL)|(3<<UCSZ0);					// Format setzen: 8 Data, 1 Stop bit, kein Parity
}

// ########## USART Transmit ##########
void USART_Transmit (char *data) {
int i = 0;
	while (data[i] != '\0') {
		while ( !( UCSRA & (1<<UDRE)) ) {;}		// Warte auf leeren Transmit Buffer
		UDR = data[i];								// Daten zum Puffer und Daten senden
		i++;
	}
}

// ########## USART Receive ##########
void USART_Receive (void) {
char cData;
	if (UCSRA & (1<<RXC)) {
		cData = UDR;
		uiUSARTBusyCounter = 32768;
		if (cData == '' && iUSARTBufferPointer == 0) {	// Start empfangen
			sUSARTBuffer[iUSARTBufferPointer] = cData;
			iUSARTBufferPointer++;
		}
		else {
			if ( iUSARTBufferPointer > 0) {
				sUSARTBuffer[iUSARTBufferPointer] = cData;
				iUSARTBufferPointer++;
				if (iUSARTBufferPointer > 135) {
					iUSARTBufferPointer = 0;
				}
			}
		}
	}
}

// ########## Execute Command ##########
void USART_Exec_Commands (void) {
unsigned char ucIdx;
unsigned char ucChk;
unsigned long lWrAdr;
	if (iUSARTBufferPointer > 2) {
		// Command Identify Device
		if (sUSARTBuffer[1] == 'I' && sUSARTBuffer[2] == 'D') {
			USART_Transmit("PRORGB\r\n");
			sUSARTBuffer[1] = 0;
			sUSARTBuffer[2] = 0;
			iUSARTBufferPointer = 0;
		}
		// Command WRite data
		if (sUSARTBuffer[1] == 'W' && sUSARTBuffer[2] == 'R' && sUSARTBuffer[134] == '#') {
			ucChk = 0xAA;
			lWrAdr = 0;
			lWrAdr |= (unsigned char)sUSARTBuffer[3];
			lWrAdr <<= 8;
			ucChk -= sUSARTBuffer[3];
			lWrAdr |= (unsigned char)sUSARTBuffer[4];
			ucChk -= sUSARTBuffer[4];
			for (ucIdx=0; ucIdx<128; ucIdx++) {
				ucEEWriteBuffer[ucIdx] = (sUSARTBuffer[ucIdx+5]);
				ucChk -= ucEEWriteBuffer[ucIdx];
			}
			sUSARTBuffer[1] = 0;
			sUSARTBuffer[2] = 0;
			sUSARTBuffer[134] = 0;
			iUSARTBufferPointer = 0;
			if (sUSARTBuffer[133]!= ucChk) {
				USART_Transmit("ERR\r\n");
				return;
			}
			USART_Transmit("OK\r\n");
			EE_Write (lWrAdr << 7);
		}
	}
}

// ########## Analogwert aufnehmen ##########
void Sample_ADC (void) {
	ADMUX  = (0<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(1<<MUX1)|(0<<MUX0);
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(0<<ADFR)|(0<<ADIF)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}